@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 #
 # __init__.py
 #
-# Copyright (C) 2017-2021 frnmst (Franco Masotti) <franco.masotti@live.com>
+# Copyright (C) 2017-2021 Franco Masotti (franco \D\o\T masotti {-A-T-} tutanota \D\o\T com)
 #
 # This file is part of fpyutils.
 #
@@ -21,8 +22,11 @@
 """Exposed API."""
 
 from .exceptions import LineOutOfFileBoundsError, NegativeLineRangeError
-from .filelines import (get_line_matches, insert_string_at_line,
-                        remove_line_interval)
+from .filelines import (
+    get_line_matches,
+    insert_string_at_line,
+    remove_line_interval,
+)
 from .notify import send_email, send_gotify_message
 from .path import add_trailing_slash, gen_pseudorandom_path
 from .shell import execute_command_live_output
